from command_parser import logger

from command_parser.flag import Flag
from command_parser.value import Value


def parse(line: str, flags: list[Flag], values: list[Value]):
    """
    parse a command line parser
    """
    logger.info("parsing a line")
    elements = line.split(" ")
    index = 0
    while index < len(elements):
        prev_index = index
        element = elements[index]
        logger.debug(
            _("parsing element {element}".format(element=element))
        )
        try:
            if element[0] != "-": # don't start with -
                for flag in flags:
                    if flag.one() in element:
                        flag.apply()
                        if len(element) == 1:
                            index += 1
                        else:
                            elements[index] = element.replace(
                                flag.one(), ""
                            )
                for value in values:
                    if value.one() in element:
                        if value.one() != element[0]:
                            continue # wait until it's the first one to pick the good associated value (hacky but work)
                        value.apply(elements.pop(index + 1))
                        if len(element) == 1:
                            index += 1 # element already removed with pop
                        else:
                            elements[index] = element.replace(
                                value.one(), ""
                            )
            elif element[1] != "-": # start with one -
                for flag in flags:
                    if flag.one() in element:
                        flag.apply()
                        if len(element) == 2:
                            index += 1
                        else:
                            elements[index] = element.replace(
                                flag.one(), ""
                            )
                for value in values:
                    if value.one() in element:
                        if value.one() != element[1]:
                            continue # wait until it's the first one to pick the good associated value (hacky but work)
                        value.apply(elements.pop(index + 1))
                        if len(element) == 2:
                            index += 1
                        else:
                            elements[index] = element.replace(
                                value.one(), ""
                            )
            else: # start with more than one -
                for flag in flags:
                    if flag.long() == element:
                        flag.apply()
                        index += 1
                for value in values:
                    if value.long() == element[: len(value.long())]:
                        value.apply(element[len(value.long()) :])
                        index += 1
            if index == prev_index:
                logger.warning(_("unknown {element}, ignoring").format(
                    element = element
                ))
                index += 1
        except IndexError:
            logger.warning(_("empty value"))
            index += 1
